@@ -1,4 +1,5 @@
 /* Overlay/Transparency
+ * #Keywords: GtkOverlay, GtkSnapshot
  *
  * Blur the background behind an overlay.
  */
@@ -60,7 +61,7 @@ do_transparent (GtkWidget *do_widget)
     }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show (window);
+    gtk_widget_set_visible (window, TRUE);
   else
     gtk_window_destroy (GTK_WINDOW (window));
 
