@@ -32,10 +32,6 @@
 #include "gsk/vulkan/gskvulkanrenderer.h"
 #endif
 
-#ifndef NODE_EDITOR_SOURCE_DIR
-#define NODE_EDITOR_SOURCE_DIR "." /* Fallback */
-#endif
-
 typedef struct
 {
   gsize  start_chars;
@@ -573,10 +569,10 @@ show_open_filechooser (NodeEditorWindow *self)
   dialog = gtk_file_dialog_new ();
   gtk_file_dialog_set_title (dialog, "Open node file");
   cwd = g_file_new_for_path (".");
-  gtk_file_dialog_set_current_folder (dialog, cwd);
+  gtk_file_dialog_set_initial_folder (dialog, cwd);
   g_object_unref (cwd);
   gtk_file_dialog_open (dialog, GTK_WINDOW (self),
-                        NULL, NULL, open_response_cb, self);
+                        NULL, open_response_cb, self);
   g_object_unref (dialog);
 }
 
@@ -637,13 +633,12 @@ save_cb (GtkWidget        *button,
   dialog = gtk_file_dialog_new ();
   gtk_file_dialog_set_title (dialog, "Save node");
   cwd = g_file_new_for_path (".");
-  gtk_file_dialog_set_current_folder (dialog, cwd);
+  gtk_file_dialog_set_initial_folder (dialog, cwd);
+  gtk_file_dialog_set_initial_name (dialog, "demo.node");
   g_object_unref (cwd);
   gtk_file_dialog_save (dialog,
                         GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button))),
                         NULL,
-                        "demo.node",
-                        NULL,
                         save_response_cb, self);
   g_object_unref (dialog);
 }
@@ -746,11 +741,10 @@ export_image_cb (GtkWidget        *button,
 
   dialog = gtk_file_dialog_new ();
   gtk_file_dialog_set_title (dialog, "");
+  gtk_file_dialog_set_initial_name (dialog, "example.png");
   gtk_file_dialog_save (dialog,
                         GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button))),
                         NULL,
-                        "example.png",
-                        NULL,
                         export_image_response_cb, texture);
   g_object_unref (dialog);
 }
@@ -788,12 +782,52 @@ testcase_name_entry_changed_cb (GtkWidget        *button,
     gtk_widget_set_sensitive (self->testcase_save_button, FALSE);
 }
 
+/* Returns the location where gsk test cases are stored in
+ * the GTK testsuite, if we can determine it.
+ *
+ * When running node editor outside of a GTK build, you can
+ * set GTK_SOURCE_DIR to point it at the checkout.
+ */
+static char *
+get_source_dir (void)
+{
+  const char *subdir = "testsuite/gsk/compare";
+  const char *source_dir;
+  char *current_dir;
+  char *dir;
+
+  source_dir = g_getenv ("GTK_SOURCE_DIR");
+  current_dir = g_get_current_dir ();
+
+  if (source_dir)
+    {
+      char *abs_source_dir = g_canonicalize_filename (source_dir, NULL);
+      dir = g_canonicalize_filename (subdir, abs_source_dir);
+      g_free (abs_source_dir);
+    }
+  else
+    {
+      dir = g_canonicalize_filename (subdir, current_dir);
+    }
+
+  if (g_file_test (dir, G_FILE_TEST_EXISTS))
+    {
+      g_free (current_dir);
+
+      return dir;
+    }
+
+  g_free (dir);
+
+  return current_dir;
+}
+
 static void
 testcase_save_clicked_cb (GtkWidget        *button,
                           NodeEditorWindow *self)
 {
   const char *testcase_name = gtk_editable_get_text (GTK_EDITABLE (self->testcase_name_entry));
-  char *source_dir = g_canonicalize_filename (NODE_EDITOR_SOURCE_DIR, NULL);
+  char *source_dir = get_source_dir ();
   char *node_file_name;
   char *node_file;
   char *png_file_name;
@@ -806,6 +840,8 @@ testcase_save_clicked_cb (GtkWidget        *button,
   node_file = g_build_filename (source_dir, node_file_name, NULL);
   g_free (node_file_name);
 
+  g_debug ("Saving testcase in %s", node_file);
+
   png_file_name = g_strconcat (testcase_name, ".png", NULL);
   png_file = g_build_filename (source_dir, png_file_name, NULL);
   g_free (png_file_name);
