@@ -60,7 +60,7 @@ gboolean            broadway_server_has_client                (BroadwayServer  *
 void                broadway_server_flush                     (BroadwayServer  *server);
 void                broadway_server_sync                      (BroadwayServer  *server);
 void                broadway_server_roundtrip                 (BroadwayServer  *server,
-                                                               gint             id,
+                                                               int              id,
                                                                guint32          tag);
 void                broadway_server_get_screen_size           (BroadwayServer  *server,
                                                                guint32         *width,
@@ -76,8 +76,8 @@ void                broadway_server_query_mouse               (BroadwayServer  *
                                                                gint32          *root_y,
                                                                guint32         *mask);
 guint32             broadway_server_grab_pointer              (BroadwayServer  *server,
-                                                               gint             client_id,
-                                                               gint             id,
+                                                               int              client_id,
+                                                               int              id,
                                                                gboolean         owner_events,
                                                                guint32          event_mask,
                                                                guint32          time_);
@@ -93,23 +93,23 @@ guint32             broadway_server_new_surface               (BroadwayServer  *
                                                                int              width,
                                                                int              height);
 void                broadway_server_destroy_surface           (BroadwayServer  *server,
-                                                               gint             id);
+                                                               int              id);
 gboolean            broadway_server_surface_show              (BroadwayServer  *server,
-                                                               gint             id);
+                                                               int              id);
 gboolean            broadway_server_surface_hide              (BroadwayServer  *server,
-                                                               gint             id);
+                                                               int              id);
 void                broadway_server_surface_raise             (BroadwayServer  *server,
-                                                               gint             id);
+                                                               int              id);
 void                broadway_server_surface_lower             (BroadwayServer  *server,
-                                                               gint             id);
+                                                               int              id);
 void                broadway_server_surface_set_transient_for (BroadwayServer  *server,
-                                                               gint             id,
-                                                               gint             parent);
+                                                               int              id,
+                                                               int              parent);
 gboolean            broadway_server_surface_translate         (BroadwayServer  *server,
-                                                               gint             id,
+                                                               int              id,
                                                                cairo_region_t  *area,
-                                                               gint             dx,
-                                                               gint             dy);
+                                                               int              dx,
+                                                               int              dy);
 guint32             broadway_server_upload_texture            (BroadwayServer  *server,
                                                                GBytes          *texture);
 void                broadway_server_release_texture           (BroadwayServer  *server,
@@ -117,19 +117,22 @@ void                broadway_server_release_texture           (BroadwayServer  *
 cairo_surface_t   * broadway_server_create_surface            (int              width,
                                                                int              height);
 void                broadway_server_surface_update_nodes      (BroadwayServer  *server,
-                                                               gint             id,
+                                                               int              id,
                                                                guint32          data[],
                                                                int              len,
                                                                GHashTable      *client_texture_map);
 gboolean            broadway_server_surface_move_resize       (BroadwayServer  *server,
-                                                               gint             id,
+                                                               int              id,
                                                                gboolean         with_move,
                                                                int              x,
                                                                int              y,
                                                                int              width,
                                                                int              height);
 void                broadway_server_focus_surface             (BroadwayServer  *server,
-                                                               gint             new_focused_surface);
+                                                               int              new_focused_surface);
+void                broadway_server_surface_set_modal_hint    (BroadwayServer *server,
+                                                               int             id,
+                                                               gboolean        modal_hint);
 
 
 #endif /* __BROADWAY_SERVER__ */
