@@ -28,7 +28,7 @@
 #include "gdkdisplaymanagerprivate.h"
 #include "gdkdisplayprivate.h"
 #include "gdkkeysprivate.h"
-#include "gdkintl.h"
+#include <glib/gi18n-lib.h>
 
 #ifdef GDK_WINDOWING_X11
 #include "x11/gdkx.h"
@@ -285,7 +285,7 @@ static GdkBackend gdk_backends[] = {
  * with multiple backends).
  *
  * Applications can use [func@set_allowed_backends] to limit what
- * backends wil be used.
+ * backends will be used.
  *
  * Returns: (transfer none): The global `GdkDisplayManager` singleton
  */
@@ -420,7 +420,7 @@ gdk_display_manager_open_display (GdkDisplayManager *manager,
               (any && strstr (allowed_backends, gdk_backends[j].name)) ||
               g_str_equal (backend, gdk_backends[j].name))
             {
-              GDK_NOTE (MISC, g_message ("Trying %s backend", gdk_backends[j].name));
+              GDK_DEBUG (MISC, "Trying %s backend", gdk_backends[j].name);
               display = gdk_backends[j].open_display (name);
               if (display)
                 break;
