@@ -1,5 +1,7 @@
+#include "config.h"
+
 #include "gskdebugprivate.h"
-#include "gdk/gdk-private.h"
+#include "gdk/gdkprivate.h"
 
 static const GdkDebugKey gsk_debug_keys[] = {
   { "renderer", GSK_DEBUG_RENDERER, "General renderer information" },
@@ -22,7 +24,7 @@ static guint gsk_debug_flags;
 static void
 init_debug_flags (void)
 {
-  static volatile gsize gsk_debug_flags__set;
+  static gsize gsk_debug_flags__set;
 
   if (g_once_init_enter (&gsk_debug_flags__set))
     {
