@@ -246,7 +246,7 @@ collect_states (GtkAtSpiContext    *self,
         case GTK_ACCESSIBLE_INVALID_TRUE:
         case GTK_ACCESSIBLE_INVALID_GRAMMAR:
         case GTK_ACCESSIBLE_INVALID_SPELLING:
-          set_atspi_state (&states, ATSPI_STATE_INVALID);
+          set_atspi_state (&states, ATSPI_STATE_INVALID_ENTRY);
           break;
         case GTK_ACCESSIBLE_INVALID_FALSE:
         default:
@@ -282,6 +282,34 @@ collect_states (GtkAtSpiContext    *self,
         }
     }
 
+  if (gtk_at_context_has_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_REQUIRED))
+    {
+      value = gtk_at_context_get_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_REQUIRED);
+      if (gtk_boolean_accessible_value_get (value))
+        set_atspi_state (&states, ATSPI_STATE_REQUIRED);
+    }
+
+  if (gtk_at_context_has_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_MULTI_SELECTABLE))
+    {
+      value = gtk_at_context_get_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_MULTI_SELECTABLE);
+      if (gtk_boolean_accessible_value_get (value))
+        set_atspi_state (&states, ATSPI_STATE_MULTISELECTABLE);
+    }
+
+  if (gtk_at_context_has_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_HAS_POPUP))
+    {
+      value = gtk_at_context_get_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_HAS_POPUP);
+      if (gtk_boolean_accessible_value_get (value))
+        set_atspi_state (&states, ATSPI_STATE_HAS_POPUP);
+    }
+
+  if (gtk_at_context_has_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_AUTOCOMPLETE))
+    {
+      value = gtk_at_context_get_accessible_property (ctx, GTK_ACCESSIBLE_PROPERTY_AUTOCOMPLETE);
+      if (gtk_autocomplete_accessible_value_get (value) != GTK_ACCESSIBLE_AUTOCOMPLETE_NONE)
+        set_atspi_state (&states, ATSPI_STATE_SUPPORTS_AUTOCOMPLETION);
+    }
+
   g_variant_builder_add (builder, "u", (guint32) (states & 0xffffffff));
   g_variant_builder_add (builder, "u", (guint32) (states >> 32));
 }
@@ -299,11 +327,11 @@ collect_relations (GtkAtSpiContext *self,
     { GTK_ACCESSIBLE_RELATION_LABELLED_BY, ATSPI_RELATION_LABELLED_BY },
     { GTK_ACCESSIBLE_RELATION_CONTROLS, ATSPI_RELATION_CONTROLLER_FOR },
     { GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, ATSPI_RELATION_DESCRIBED_BY },
+    { GTK_ACCESSIBLE_RELATION_DETAILS, ATSPI_RELATION_DETAILS },
     { GTK_ACCESSIBLE_RELATION_FLOW_TO, ATSPI_RELATION_FLOWS_TO},
   };
   GtkAccessibleValue *value;
   GList *list, *l;
-  GtkATContext *target_ctx;
   int i;
 
   for (i = 0; i < G_N_ELEMENTS (map); i++)
@@ -318,13 +346,16 @@ collect_relations (GtkAtSpiContext *self,
 
       for (l = list; l; l = l->next)
         {
-          target_ctx = gtk_accessible_get_at_context (GTK_ACCESSIBLE (l->data));
+          GtkATContext *target_ctx =
+            gtk_accessible_get_at_context (GTK_ACCESSIBLE (l->data));
 
           /* Realize the ATContext of the target, so we can ask for its ref */
           gtk_at_context_realize (target_ctx);
 
           g_variant_builder_add (&b, "@(so)",
                                  gtk_at_spi_context_to_ref (GTK_AT_SPI_CONTEXT (target_ctx)));
+
+          g_object_unref (target_ctx);
         }
 
       g_variant_builder_add (builder, "(ua(so))", map[i].s, &b);
@@ -336,17 +367,17 @@ static int
 get_index_in (GtkAccessible *parent,
               GtkAccessible *child)
 {
-  GtkAccessible *candidate;
-  guint res;
-
   if (parent == NULL)
     return -1;
 
-  res = 0;
+  guint res = 0;
+  GtkAccessible *candidate;
   for (candidate = gtk_accessible_get_first_accessible_child (parent);
        candidate != NULL;
        candidate = gtk_accessible_get_next_accessible_sibling (candidate))
     {
+      g_object_unref (candidate);
+
       if (candidate == child)
         return res;
 
@@ -365,7 +396,13 @@ get_index_in_parent (GtkAccessible *accessible)
   GtkAccessible *parent = gtk_accessible_get_accessible_parent (accessible);
 
   if (parent != NULL)
-    return get_index_in (parent, accessible);
+    {
+      int res = get_index_in (parent, accessible);
+
+      g_object_unref (parent);
+
+      return res;
+    }
 
   return -1;
 }
@@ -401,7 +438,6 @@ static GVariant *
 get_parent_context_ref (GtkAccessible *accessible)
 {
   GVariant *res = NULL;
-
   GtkAccessible *parent = gtk_accessible_get_accessible_parent (accessible);
 
   if (parent == NULL)
@@ -410,13 +446,19 @@ get_parent_context_ref (GtkAccessible *accessible)
       GtkAtSpiContext *self = GTK_AT_SPI_CONTEXT (context);
 
       res = gtk_at_spi_root_to_ref (self->root);
+
+      g_object_unref (context);
     }
   else
     {
       GtkATContext *parent_context = gtk_accessible_get_at_context (parent);
+
       gtk_at_context_realize (parent_context);
 
       res = gtk_at_spi_context_to_ref (GTK_AT_SPI_CONTEXT (parent_context));
+
+      g_object_unref (parent_context);
+      g_object_unref (parent);
     }
 
   if (res == NULL)
@@ -497,31 +539,32 @@ handle_accessible_method (GDBusConnection       *connection,
     {
       GtkATContext *context = NULL;
       GtkAccessible *accessible;
+      GtkAccessible *child = NULL;
       int idx, presentable_idx;
 
       g_variant_get (parameters, "(i)", &idx);
 
       accessible = gtk_at_context_get_accessible (GTK_AT_CONTEXT (self));
 
-      GtkAccessible *child;
-
       presentable_idx = 0;
+
       for (child = gtk_accessible_get_first_accessible_child (accessible);
            child != NULL;
            child = gtk_accessible_get_next_accessible_sibling (child))
         {
+          g_object_unref (child);
+
           if (!gtk_accessible_should_present (child))
-              continue;
+            continue;
 
           if (presentable_idx == idx)
             break;
-          presentable_idx++;
 
+          presentable_idx += 1;
         }
-      if (child)
-        {
-          context = gtk_accessible_get_at_context (child);
-        }
+
+      if (child != NULL)
+        context = gtk_accessible_get_at_context (child);
 
       if (context == NULL)
         {
@@ -536,20 +579,23 @@ handle_accessible_method (GDBusConnection       *connection,
       gtk_at_context_realize (context);
 
       GVariant *ref = gtk_at_spi_context_to_ref (GTK_AT_SPI_CONTEXT (context));
-
       g_dbus_method_invocation_return_value (invocation, g_variant_new ("(@(so))", ref));
+
+      g_object_unref (context);
     }
   else if (g_strcmp0 (method_name, "GetChildren") == 0)
     {
       GVariantBuilder builder = G_VARIANT_BUILDER_INIT (G_VARIANT_TYPE ("a(so)"));
 
       GtkAccessible *accessible = gtk_at_context_get_accessible (GTK_AT_CONTEXT (self));
+      GtkAccessible *child = NULL;
 
-      GtkAccessible *child;
       for (child = gtk_accessible_get_first_accessible_child (accessible);
            child != NULL;
            child = gtk_accessible_get_next_accessible_sibling (child))
-      {
+        {
+          g_object_unref (child);
+
           if (!gtk_accessible_should_present (child))
             continue;
 
@@ -562,6 +608,8 @@ handle_accessible_method (GDBusConnection       *connection,
 
           if (ref != NULL)
             g_variant_builder_add (&builder, "@(so)", ref);
+
+          g_object_unref (context);
         }
 
       g_dbus_method_invocation_return_value (invocation, g_variant_new ("(a(so))", &builder));
@@ -741,8 +789,13 @@ emit_property_changed (GtkAtSpiContext *self,
                        const char      *name,
                        GVariant        *value)
 {
+  GVariant *value_owned = g_variant_ref_sink (value);
+
   if (self->connection == NULL)
-    return;
+    {
+      g_variant_unref (value_owned);
+      return;
+    }
 
   g_dbus_connection_emit_signal (self->connection,
                                  NULL,
@@ -750,8 +803,9 @@ emit_property_changed (GtkAtSpiContext *self,
                                  "org.a11y.atspi.Event.Object",
                                  "PropertyChange",
                                  g_variant_new ("(siiva{sv})",
-                                                name, 0, 0, value, NULL),
+                                                name, 0, 0, value_owned, NULL),
                                  NULL);
+  g_variant_unref (value_owned);
 }
 
 static void
@@ -850,8 +904,6 @@ gtk_at_spi_context_state_change (GtkATContext                *ctx,
 
   if (changed_states & GTK_ACCESSIBLE_STATE_CHANGE_HIDDEN)
     {
-      GtkAccessible *parent;
-      GtkATContext *context;
       GtkAccessibleChildChange change;
 
       value = gtk_accessible_attribute_set_get_value (states, GTK_ACCESSIBLE_STATE_HIDDEN);
@@ -867,10 +919,15 @@ gtk_at_spi_context_state_change (GtkATContext                *ctx,
         }
       else
         {
-          parent = gtk_accessible_get_accessible_parent (accessible);
+          GtkAccessible *parent =
+            gtk_accessible_get_accessible_parent (accessible);
+          GtkATContext *context =
+            gtk_accessible_get_at_context (parent);
 
-          context = gtk_accessible_get_at_context (parent);
           gtk_at_context_child_changed (context, change, accessible);
+
+          g_object_unref (context);
+          g_object_unref (parent);
         }
     }
 
@@ -1119,9 +1176,18 @@ gtk_at_spi_context_child_change (GtkATContext             *ctx,
   int idx = 0;
 
   if (parent == NULL)
-    idx = -1;
+    {
+      idx = -1;
+    }
   else if (parent == accessible)
-    idx = get_index_in (accessible, child);
+    {
+      idx = get_index_in (accessible, child);
+      g_object_unref (parent);
+    }
+  else
+    {
+      g_object_unref (parent);
+    }
 
   if (change & GTK_ACCESSIBLE_CHILD_CHANGE_ADDED)
     emit_children_changed (self,
@@ -1133,6 +1199,8 @@ gtk_at_spi_context_child_change (GtkATContext             *ctx,
                            GTK_AT_SPI_CONTEXT (child_context),
                            idx,
                            GTK_ACCESSIBLE_CHILD_STATE_REMOVED);
+
+  g_object_unref (child_context);
 }
 /* }}} */
 /* {{{ D-Bus Registration */
@@ -1701,15 +1769,16 @@ gtk_at_spi_context_get_child_count (GtkAtSpiContext *self)
   int n_children = 0;
 
   GtkAccessible *child = NULL;
-
   for (child = gtk_accessible_get_first_accessible_child (accessible);
        child != NULL;
        child = gtk_accessible_get_next_accessible_sibling (child))
     {
+      g_object_unref (child);
+
       if (!gtk_accessible_should_present (child))
         continue;
 
-      n_children++;
+      n_children += 1;
     }
 
   return n_children;
