@@ -34,9 +34,13 @@ G_DECLARE_FINAL_TYPE (GtkAtSpiRoot, gtk_at_spi_root, GTK, AT_SPI_ROOT, GObject)
 GtkAtSpiRoot *
 gtk_at_spi_root_new (const char *bus_address);
 
+typedef void (* GtkAtSpiRootRegisterFunc) (GtkAtSpiRoot *root,
+                                           GtkAtSpiContext *context);
+
 void
 gtk_at_spi_root_queue_register (GtkAtSpiRoot *self,
-                                GtkAtSpiContext *context);
+                                GtkAtSpiContext *context,
+                                GtkAtSpiRootRegisterFunc func);
 
 void
 gtk_at_spi_root_unregister (GtkAtSpiRoot *self,
