@@ -22,6 +22,7 @@
 #include "gtkalertdialog.h"
 
 #include "gtkbutton.h"
+#include "gtkdialogerror.h"
 #include "deprecated/gtkmessagedialog.h"
 #include <glib/gi18n-lib.h>
 
@@ -462,7 +463,7 @@ gtk_alert_dialog_set_detail (GtkAlertDialog *self,
  *
  * Returns the button labels for the alert.
  *
- * Returns: (nullable) (transfer none): the button labels
+ * Returns: (nullable) (transfer none) (array zero-terminated=1): the button labels
  *
  * Since: 4.10
  */
@@ -477,7 +478,7 @@ gtk_alert_dialog_get_buttons (GtkAlertDialog *self)
 /**
  * gtk_alert_dialog_set_buttons:
  * @self: a `GtkAlertDialog`
- * @labels: the new button labels
+ * @labels: (array zero-terminated=1): the new button labels
  *
  * Sets the button labels for the alert.
  *
@@ -606,7 +607,11 @@ response_cb (GTask *task,
   if (cancellable)
     g_signal_handlers_disconnect_by_func (cancellable, cancelled_cb, task);
 
-  if (response >= 0)
+  if (response == GTK_RESPONSE_CLOSE)
+    {
+      g_task_return_new_error (task, GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_CANCELLED, "Cancelled by application");
+    }
+  else if (response >= 0)
     {
       g_task_return_int (task, response);
     }
@@ -614,7 +619,10 @@ response_cb (GTask *task,
     {
       GtkAlertDialog *self = GTK_ALERT_DIALOG (g_task_get_source_object (task));
 
-      g_task_return_int (task, self->cancel_return);
+      if (self->cancel_return >= 0)
+        g_task_return_int (task, self->cancel_return);
+      else
+        g_task_return_new_error (task, GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_DISMISSED, "Dismissed by user");
     }
 
   g_object_unref (task);
@@ -719,6 +727,7 @@ gtk_alert_dialog_choose (GtkAlertDialog      *self,
  * gtk_alert_dialog_choose_finish:
  * @self: a `GtkAlertDialog`
  * @result: a `GAsyncResult`
+ * @error: return location for a [enum@Gtk.DialogError] error
  *
  * Finishes the [method@Gtk.AlertDialog.choose] call
  * and returns the index of the button that was clicked.
@@ -731,13 +740,14 @@ gtk_alert_dialog_choose (GtkAlertDialog      *self,
  */
 int
 gtk_alert_dialog_choose_finish (GtkAlertDialog  *self,
-                                GAsyncResult   *result)
+                                GAsyncResult    *result,
+                                GError         **error)
 {
   g_return_val_if_fail (GTK_IS_ALERT_DIALOG (self), -1);
   g_return_val_if_fail (g_task_is_valid (result, self), -1);
   g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gtk_alert_dialog_choose, -1);
 
-  return (int) g_task_propagate_int (G_TASK (result), NULL);
+  return (int) g_task_propagate_int (G_TASK (result), error);
 }
 
 /**
@@ -745,11 +755,13 @@ gtk_alert_dialog_choose_finish (GtkAlertDialog  *self,
  * @self: a `GtkAlertDialog`
  * @parent: (nullable): the parent `GtkWindow`
  *
- * This function shows the alert to the user.
+ * Show the alert to the user.
  *
- * If the alert has more than one button, you should use
- * [method@Gtk.AlertDialog.choose] instead and provide
- * a callback that can react to the button that was clicked.
+ * This function is a simple version of [method@Gtk.AlertDialog.choose]
+ * intended for dialogs with a single button.
+ * If you want to cancel the dialog or if the alert has more than one button,
+ * you should use that function instead and provide it with a #GCancellable or
+ * callback respectively.
  *
  * Since: 4.10
  */
