@@ -17,17 +17,18 @@
 
 #include "config.h"
 
-#include "gtkdialog.h"
-#include "gtkdialogprivate.h"
+#include "deprecated/gtkdialog.h"
+#include "deprecated/gtkdialogprivate.h"
 #include "gtkbutton.h"
 #include "gtkbox.h"
 #include "gtkprivate.h"
-#include "gtkintl.h"
 #include "gtksettings.h"
 
-#include "gtkcolorchooserprivate.h"
-#include "gtkcolorchooserdialog.h"
-#include "gtkcolorchooserwidget.h"
+#include "deprecated/gtkcolorchooserprivate.h"
+#include "deprecated/gtkcolorchooserdialog.h"
+#include "deprecated/gtkcolorchooserwidget.h"
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
 /**
  * GtkColorChooserDialog:
@@ -44,6 +45,10 @@
  * To change the initially selected color, use
  * [method@Gtk.ColorChooser.set_rgba]. To get the selected color use
  * [method@Gtk.ColorChooser.get_rgba].
+ *
+ * `GtkColorChooserDialog` has been deprecated in favor of [class@Gtk.ColorDialog].
+ *
+ * Deprecated: 4.10: Use [class@Gtk.ColorDialog] instead
  */
 
 typedef struct _GtkColorChooserDialogClass   GtkColorChooserDialogClass;
@@ -223,7 +228,7 @@ gtk_color_chooser_dialog_class_init (GtkColorChooserDialogClass *class)
   g_object_class_override_property (object_class, PROP_RGBA, "rgba");
   g_object_class_override_property (object_class, PROP_USE_ALPHA, "use-alpha");
   g_object_class_install_property (object_class, PROP_SHOW_EDITOR,
-      g_param_spec_boolean ("show-editor", P_("Show editor"), P_("Show editor"),
+      g_param_spec_boolean ("show-editor", NULL, NULL,
                             FALSE, GTK_PARAM_READWRITE));
 
   /* Bind class to template
@@ -282,6 +287,8 @@ gtk_color_chooser_dialog_iface_init (GtkColorChooserInterface *iface)
  * Creates a new `GtkColorChooserDialog`.
  *
  * Returns: a new `GtkColorChooserDialog`
+ *
+ * Deprecated: 4.10: Use [class@Gtk.ColorDialog] instead
  */
 GtkWidget *
 gtk_color_chooser_dialog_new (const char *title,
