@@ -23,7 +23,6 @@
 #include "gtkcssimageprivate.h"
 #include "gtkcsspalettevalueprivate.h"
 #include "gtkcsscolorvalueprivate.h"
-#include "gtkiconthemeprivate.h"
 #include "gdkpixbufutilsprivate.h"
 
 #include "gtkstyleproviderprivate.h"
@@ -130,7 +129,7 @@ static GtkCssImage *
 gtk_css_image_recolor_load (GtkCssImageRecolor  *recolor,
                             GtkCssStyle         *style,
                             GtkCssValue         *palette,
-                            gint                 scale,
+                            int                  scale,
                             GError             **gerror)
 {
   GError *local_error = NULL;
