@@ -27,7 +27,6 @@
 #include "gtkdropcontrollermotion.h"
 #include "gtkgesturedrag.h"
 #include "gtkgizmoprivate.h"
-#include "gtkintl.h"
 #include "gtklistitemwidgetprivate.h"
 #include "gtkmultiselection.h"
 #include "gtkorientable.h"
@@ -379,10 +378,10 @@ gtk_list_base_get_allocation_across (GtkListBase *self,
  * @pos: item to select
  * @modify: %TRUE if the selection should be modified, %FALSE
  *   if a new selection should be done. This is usually set
- *   to %TRUE if the user keeps the <Shift> key pressed.
+ *   to %TRUE if the user keeps the `<Shift>` key pressed.
  * @extend_pos: %TRUE if the selection should be extended.
  *   Selections are usually extended from the last selected
- *   position if the user presses the <Ctrl> key.
+ *   position if the user presses the `<Ctrl>` key.
  *
  * Selects the item at @pos according to how GTK list widgets modify
  * selections, both when clicking rows with the mouse or when using
@@ -570,6 +569,20 @@ gtk_list_base_focus (GtkWidget        *widget,
   return gtk_widget_child_focus (item->widget, direction);
 }
 
+static gboolean
+gtk_list_base_grab_focus (GtkWidget *widget)
+{
+  GtkListBase *self = GTK_LIST_BASE (widget);
+  GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
+  guint pos;
+
+  pos = gtk_list_item_tracker_get_position (priv->item_manager, priv->focus);
+  if (gtk_list_base_grab_focus_on_item (self, pos, FALSE, FALSE, FALSE))
+    return TRUE;
+
+  return GTK_WIDGET_CLASS (gtk_list_base_parent_class)->grab_focus (widget);
+}
+
 static void
 gtk_list_base_dispose (GObject *object)
 {
@@ -790,43 +803,13 @@ gtk_list_base_compute_scroll_align (GtkListBase   *self,
 }
 
 static void
-gtk_list_base_update_focus_tracker (GtkListBase *self)
+gtk_list_base_scroll_to_item (GtkListBase *self,
+                              guint        pos)
 {
-  GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
-  GtkWidget *focus_child;
-  guint pos;
-
-  focus_child = gtk_widget_get_focus_child (GTK_WIDGET (self));
-  if (!GTK_IS_LIST_ITEM_WIDGET (focus_child))
-    return;
-
-  pos = gtk_list_item_widget_get_position (GTK_LIST_ITEM_WIDGET (focus_child));
-  if (pos != gtk_list_item_tracker_get_position (priv->item_manager, priv->focus))
-    {
-      gtk_list_item_tracker_set_position (priv->item_manager,
-                                          priv->focus,
-                                          pos,
-                                          0,
-                                          0);
-    }
-}
-
-static void
-gtk_list_base_scroll_to_item (GtkWidget  *widget,
-                              const char *action_name,
-                              GVariant   *parameter)
-{
-  GtkListBase *self = GTK_LIST_BASE (widget);
   GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
   int start, end;
   double align_along, align_across;
   GtkPackType side_along, side_across;
-  guint pos;
-
-  if (!g_variant_check_format_string (parameter, "u", FALSE))
-    return;
-
-  g_variant_get (parameter, "u", &pos);
 
   /* figure out primary orientation and if position is valid */
   if (!gtk_list_base_get_allocation_along (GTK_LIST_BASE (self), pos, &start, &end))
@@ -854,14 +837,48 @@ gtk_list_base_scroll_to_item (GtkWidget  *widget,
                             pos,
                             align_across, side_across,
                             align_along, side_along);
+}
 
-  /* HACK HACK HACK
-   *
-   * GTK has no way to track the focused child. But we now that when a listitem
-   * gets focus, it calls this action. So we update our focus tracker from here
-   * because it's the closest we can get to accurate tracking.
-   */
-  gtk_list_base_update_focus_tracker (self);
+static void
+gtk_list_base_scroll_to_item_action (GtkWidget  *widget,
+                                     const char *action_name,
+                                     GVariant   *parameter)
+{
+  GtkListBase *self = GTK_LIST_BASE (widget);
+  guint pos;
+
+  if (!g_variant_check_format_string (parameter, "u", FALSE))
+    return;
+
+  g_variant_get (parameter, "u", &pos);
+
+  gtk_list_base_scroll_to_item (self, pos);
+}
+
+static void
+gtk_list_base_set_focus_child (GtkWidget *widget,
+                               GtkWidget *child)
+{
+  GtkListBase *self = GTK_LIST_BASE (widget);
+  GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
+  guint pos;
+
+  GTK_WIDGET_CLASS (gtk_list_base_parent_class)->set_focus_child (widget, child);
+
+  if (!GTK_IS_LIST_ITEM_WIDGET (child))
+    return;
+
+  pos = gtk_list_item_widget_get_position (GTK_LIST_ITEM_WIDGET (child));
+
+  if (pos != gtk_list_item_tracker_get_position (priv->item_manager, priv->focus))
+    {
+      gtk_list_base_scroll_to_item (self, pos);
+      gtk_list_item_tracker_set_position (priv->item_manager,
+                                          priv->focus,
+                                          pos,
+                                          0,
+                                          0);
+    }
 }
 
 static void
@@ -1121,6 +1138,8 @@ gtk_list_base_class_init (GtkListBaseClass *klass)
   gpointer iface;
 
   widget_class->focus = gtk_list_base_focus;
+  widget_class->grab_focus = gtk_list_base_grab_focus;
+  widget_class->set_focus_child = gtk_list_base_set_focus_child;
 
   gobject_class->dispose = gtk_list_base_dispose;
   gobject_class->get_property = gtk_list_base_get_property;
@@ -1165,7 +1184,7 @@ gtk_list_base_class_init (GtkListBaseClass *klass)
   gtk_widget_class_install_action (widget_class,
                                    "list.scroll-to-item",
                                    "u",
-                                   gtk_list_base_scroll_to_item);
+                                   gtk_list_base_scroll_to_item_action);
 
   /**
    * GtkListBase|list.select-item:
@@ -1553,25 +1572,16 @@ gtk_list_base_start_rubberband (GtkListBase *self,
 }
 
 static void
-gtk_list_base_stop_rubberband (GtkListBase *self,
-                               gboolean     modify,
-                               gboolean     extend)
+gtk_list_base_apply_rubberband_selection (GtkListBase *self,
+                                          gboolean     modify,
+                                          gboolean     extend)
 {
   GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
-  GtkListItemManagerItem *item;
   GtkSelectionModel *model;
 
   if (!priv->rubberband)
     return;
 
-  for (item = gtk_list_item_manager_get_first (priv->item_manager);
-       item != NULL;
-       item = gtk_rb_tree_node_get_next (item))
-    {
-      if (item->widget)
-        gtk_widget_unset_state_flags (item->widget, GTK_STATE_FLAG_ACTIVE);
-    }
-
   model = gtk_list_item_manager_get_model (priv->item_manager);
   if (model != NULL)
     {
@@ -1583,25 +1593,28 @@ gtk_list_base_stop_rubberband (GtkListBase *self,
         return;
 
       rubberband_selection = gtk_list_base_get_items_in_rect (self, &rect);
-      if (gtk_bitset_is_empty (rubberband_selection))
-        {
-          gtk_bitset_unref (rubberband_selection);
-          return;
-        }
 
       if (modify && extend) /* Ctrl + Shift */
         {
-          GtkBitset *current;
-          guint min = gtk_bitset_get_minimum (rubberband_selection);
-          guint max = gtk_bitset_get_maximum (rubberband_selection);
-          /* toggle the rubberband, keep the rest */
-          current = gtk_selection_model_get_selection_in_range (model, min, max - min + 1);
-          selected = gtk_bitset_copy (current);
-          gtk_bitset_unref (current);
-          gtk_bitset_intersect (selected, rubberband_selection);
-          gtk_bitset_difference (selected, rubberband_selection);
+          if (gtk_bitset_is_empty (rubberband_selection))
+            {
+              selected = gtk_bitset_ref (rubberband_selection);
+              mask = gtk_bitset_ref (rubberband_selection);
+            }
+          else
+            {
+              GtkBitset *current;
+              guint min = gtk_bitset_get_minimum (rubberband_selection);
+              guint max = gtk_bitset_get_maximum (rubberband_selection);
+              /* toggle the rubberband, keep the rest */
+              current = gtk_selection_model_get_selection_in_range (model, min, max - min + 1);
+              selected = gtk_bitset_copy (current);
+              gtk_bitset_unref (current);
+              gtk_bitset_intersect (selected, rubberband_selection);
+              gtk_bitset_difference (selected, rubberband_selection);
 
-          mask = gtk_bitset_ref (rubberband_selection);
+              mask = gtk_bitset_ref (rubberband_selection);
+            }
         }
       else if (modify) /* Ctrl */
         {
@@ -1629,6 +1642,24 @@ gtk_list_base_stop_rubberband (GtkListBase *self,
       gtk_bitset_unref (mask);
       gtk_bitset_unref (rubberband_selection);
     }
+}
+
+static void
+gtk_list_base_stop_rubberband (GtkListBase *self)
+{
+  GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
+  GtkListItemManagerItem *item;
+
+  if (!priv->rubberband)
+    return;
+
+  for (item = gtk_list_item_manager_get_first (priv->item_manager);
+       item != NULL;
+       item = gtk_rb_tree_node_get_next (item))
+    {
+      if (item->widget)
+        gtk_widget_unset_state_flags (item->widget, GTK_STATE_FLAG_ACTIVE);
+    }
 
   gtk_list_item_tracker_free (priv->item_manager, priv->rubberband->start_tracker);
   g_clear_pointer (&priv->rubberband->widget, gtk_widget_unparent);
@@ -1741,14 +1772,23 @@ gtk_list_base_drag_end (GtkGestureDrag *gesture,
                         GtkListBase    *self)
 {
   GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
+  GdkEventSequence *sequence;
   gboolean modify, extend;
 
   if (!priv->rubberband)
     return;
 
+  sequence = gtk_gesture_get_last_updated_sequence (GTK_GESTURE (gesture));
+  if (!gtk_gesture_handles_sequence (GTK_GESTURE (gesture), sequence))
+    {
+      gtk_list_base_stop_rubberband (self);
+      return;
+    }
+
   gtk_list_base_drag_update (gesture, offset_x, offset_y, self);
   get_selection_modifiers (GTK_GESTURE (gesture), &modify, &extend);
-  gtk_list_base_stop_rubberband (self, modify, extend);
+  gtk_list_base_apply_rubberband_selection (self, modify, extend);
+  gtk_list_base_stop_rubberband (self);
 }
 
 void
